package Sys::Socket::Error::InetInvalidNetworkAddress;

1;

=head1 Name

Sys::Socket::Error::InetInvalidNetworkAddress - Invalid Network address Socket Error

=head1 Description

Sys::Socket::Error class in L<SPVM> represents a socket error "The got address is not a valid network address".

=head1 Usage

  use Sys::Socket::Error::InetInvalidNetworkAddress;

=head1 Super Class

L<Error|SPVM::Error>

=head1 See Also

=over 2

=item * L<Sys|SPVM::Sys>

=back

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License

